@@ -1,4 +1,4 @@
-languages <- matrix(dimnames = list(NULL, c(
+.languages <- matrix(dimnames = list(NULL, c(
     "LANGUAGE", "Format: (* Custom Locale)"    , "locale"                      )), data = c(
     "da"      , "Danish (Denmark)"             , "Danish_Denmark"              ,
     "de"      , "German (Germany)"             , "German_Germany"              ,
@@ -21,31 +21,31 @@ languages <- matrix(dimnames = list(NULL, c(
     "zh_CN"   , "Chinese (Simplified, China)"  , "Chinese (Simplified)_China"  ,
     "zh_TW"   , "Chinese (Traditional, Taiwan)", "Chinese (Traditional)_Taiwan"
 ), ncol = 3L, byrow = TRUE)
-rownames(languages) <- languages[, "LANGUAGE"]
+rownames(.languages) <- .languages[, "LANGUAGE"]
 
 
-locales <- languages[, "locale"]
+.locales <- .languages[, "locale"]
 
 
-languageEnvvars <- function (LANGUAGE = Sys.getenv("LANGUAGE"), utf8 = identical(R.version[["crt"]], "ucrt"))
+.languageEnvvars <- function (LANGUAGE = Sys.getenv("LANGUAGE"), utf8 = identical(R.version[["crt"]], "ucrt"))
 {
     if (!is.character(LANGUAGE) || length(LANGUAGE) != 1L)
         stop(gettextf("'%s' must be a character string", "LANGUAGE", domain = "R"), domain = NA)
     if (.Platform$OS.type == "windows") {
         if (!nzchar(LANGUAGE))
             return(c("LANGUAGE=", "LC_ALL="))
-        LANGUAGE <- match.arg(LANGUAGE, c(rownames(languages), NA))
+        LANGUAGE <- match.arg(LANGUAGE, c(rownames(.languages), NA))
         if (is.na(LANGUAGE))
             return(c("LANGUAGE=", "LC_ALL="))
         paste0(
             c("LANGUAGE=", "LC_ALL="),
-            c(LANGUAGE, locales[[LANGUAGE]]),
-            if (utf8 && nzchar(locales[[LANGUAGE]])) c("", ".utf8")
+            c(LANGUAGE, .locales[[LANGUAGE]]),
+            if (utf8 && nzchar(.locales[[LANGUAGE]])) c("", ".utf8")
         )
     } else {
         if (!nzchar(LANGUAGE))
             return("LANGUAGE=")
-        LANGUAGE <- match.arg(LANGUAGE, c(rownames(languages), NA))
+        LANGUAGE <- match.arg(LANGUAGE, c(rownames(.languages), NA))
         if (is.na(LANGUAGE))
             return("LANGUAGE=")
         paste0("LANGUAGE=", LANGUAGE)
@@ -83,21 +83,21 @@ if (sys.nframe() != 0L) {
         # testing <- TRUE; warning("comment 'testing <- TRUE' out later", immediate. = TRUE)
 
 
-        # there was a time when I was doing something more along the lines:
-        #
-        # ```
-        # for (language in rownames(languages)) {
-        #     Sys.putenv(languageEnvvars(language))
-        #     gettext("Untitled", domain = "RGui")
-        #     gettext("R Editor", domain = "RGui")
-        # }
-        # ```
-        #
-        # but it doesn't work well for Microsoft Visual C++ Runtime
-        # because the messages usually get mis-translated in the windows titles
-        # but they don't with gettext()
-        #
-        # and it also doesn't work well for Universal C Runtime in Lithuanian
+        ## there was a time when I was doing something more along the lines:
+        ##
+        ## ```
+        ## for (language in rownames(.languages)) {
+        ##     Sys.putenv(.languageEnvvars(language))
+        ##     gettext("Untitled", domain = "RGui")
+        ##     gettext("R Editor", domain = "RGui")
+        ## }
+        ## ```
+        ##
+        ## but it doesn't work well for Microsoft Visual C++ Runtime
+        ## because the messages usually get mis-translated in the windows titles
+        ## but they don't with gettext()
+        ##
+        ## and it also doesn't work well for Universal C Runtime in Lithuanian
 
 
         stopifnot(.Platform$OS.type == "windows")
@@ -246,22 +246,22 @@ if (sys.nframe() != 0L) {
             Sys.setenv(R_PROFILE_USER = tmpRprofile)
 
 
-            # we want to provide --vanilla to enable factory-default settings
-            # for Rgui.exe
-            #
-            # --vanilla is a combination of --no-save, --no-restore,
-            #           --no-site-file, --no-init-file, --no-environ,
-            #           and --no-Rconsole
-            #
-            # however, we want to run the init file, so instead of --vanilla,
-            # use the other arguments except --no-init-file
+            ## we want to provide --vanilla to enable factory-default settings
+            ## for Rgui.exe
+            ##
+            ## --vanilla is a combination of --no-save, --no-restore,
+            ##           --no-site-file, --no-init-file, --no-environ,
+            ##           and --no-Rconsole
+            ##
+            ## however, we want to run the init file, so instead of --vanilla,
+            ## use the other arguments except --no-init-file
             options <- c("R_DEFAULT_PACKAGES=NULL", "--no-save", "--no-restore",
                 "--no-site-file", "--no-environ", "--no-Rconsole")
 
 
             n <- 0L
-            for (language in rownames(languages)) {
-                args <- c(rgui, options, languageEnvvars(language, ucrt))
+            for (language in rownames(.languages)) {
+                args <- c(rgui, options, .languageEnvvars(language, ucrt))
                 command <- paste(shQuote(args), collapse = " ")
                 ans <- system(command)
                 if (ans) {
@@ -366,11 +366,11 @@ if (sys.nframe() != 0L) {
             } else {
 
 
-                # string comparisons often involve translating between encodings
-                # so we will do raw comparisons instead to avoid translations
+                ## string comparisons often involve translating between encodings
+                ## so we will do raw comparisons instead to avoid translations
                 r.editor <- vapply(r.editor, function(str) {
                     bytes <- charToRaw(str)
-                    # all "R Editor" strings must start with this prefix
+                    ## all "R Editor" strings must start with this prefix
                     prefix <- c(charToRaw(tmpR), charToRaw(" - "))
                     if (length(bytes) < length(prefix) ||
                         any(bytes[seq_along(prefix)] != prefix))
@@ -391,8 +391,8 @@ if (sys.nframe() != 0L) {
 
 
                     writeLines2 <- function(x, path) {
-                        # save the text as its bytes without translation
-                        # plus its encoding
+                        ## save the text as its bytes without translation
+                        ## plus its encoding
                         conn <- file(path, "wb", encoding = "")
                         on.exit(close(conn))
                         writeLines(rbind(x, Encoding(x)), conn, sep = "\r\n", useBytes = TRUE)
